public class Column
{

	private String name;
	private String description;
	private boolean isNotNull = false;
	private String size;
	private String scale;
	private String type;
	private Table parentTable;
	private boolean isPrimaryKey = false;
	private boolean isUnique = false;
	private boolean isAutoIncrement = false;
	private String defaultValue;

	/**
	 * Creates a new instance with a <code>null</code> name.
	 */
	public Column()
	{
		this(null);
	}
	/**
	 * Creates a new column and set the name
	 * 
	 * @param name
	 *                  column name
	 */
	public Column(String name)
	{
		this.name = name;
	}
	/**
	 * Get the name of the column
	 */
	public String getName()
	{
		return name;
	}
	/**
	 * Set the name of the column
	 */
	public void setName(String newName)
	{
		name = newName;
	}
	/**
	 * Set the colunm type
	 */
	public void setType(String type)
	{
		this.type = type;
	}
	/**
	 * Get the colunm type
	 */
	public String getType()
	{
		return type;
	}

	/**
	 * Get the description for the Table
	 */
	public String getDescription()
	{
		return description;
	}
	/**
	 * Set the description for the Table
	 * 
	 * @param newDescription
	 *                  description for the Table
	 */
	public void setDescription(String newDescription)
	{
		description = newDescription;
	}
	/**
	 * Return the isNotNull property of the column
	 */
	public boolean isNotNull()
	{
		return isNotNull;
	}

	/**
	 * Set the isNotNull property of the column
	 */
	public void setNotNull(boolean status)
	{
		isNotNull = status;
	}

	/**
	 * Set if the column is a primary key or not
	 */
	public void setPrimaryKey(boolean pk)
	{
		isPrimaryKey = pk;
		// a pk is not null
		// isNotNull = true;
	}

	/**
	 * Return true if the column is a primary key
	 */
	public boolean isPrimaryKey()
	{
		return isPrimaryKey;
	}

	/**
	 * Set true if the column is UNIQUE
	 */
	public void setUnique(boolean u)
	{
		isUnique = u;
	}

	/**
	 * Get the UNIQUE property
	 */
	public boolean isUnique()
	{
		return isUnique;
	}

	/**
	 * Returns the size of the column
	 */
	public String getSize()
	{
		return size;
	}
	/**
	 * Set the size of the column
	 */
	public void setSize(String newSize)
	{
		size = newSize;
	}
	/**
	 * Returns the scale of the column
	 */
	public String getScale()
	{
		return scale;
	}
	/**
	 * Set the scale of the column
	 */
	public void setScale(String newScale)
	{
		scale = newScale;
	}
	/**
	 * Set a string that will give this column a default value.
	 */
	public void setDefaultValue(String def)
	{
		defaultValue = def;
	}
	/**
	 * Get a string that will give this column a default value.
	 */
	public String getDefaultValue()
	{
		return defaultValue;
	}
	/**
	 * Return auto increment/sequence string for the target database. We need to pass in the props for the target database!
	 */
	public boolean isAutoIncrement()
	{
		return isAutoIncrement;
	}
	/**
	 * Set the auto increment value. Use isAutoIncrement() to find out if it is set or not.
	 */
	public void setAutoIncrement(boolean value)
	{
		isAutoIncrement = value;
	}
	/**
	 * Set the parent Table of the column
	 */
	public void setTable(Table parent)
	{
		parentTable = parent;
	}
	/**
	 * Get the parent Table of the column
	 */
	public Table getTable()
	{
		return parentTable;
	}
	/**
	 * Returns the Name of the table the column is in
	 */
	public String getTableName()
	{
		return parentTable.getName();
	}	
	/**
	* String representation of the column.
	* @param numPk  is the number of part of the primary key in the table
	* @return string representation
	* 
	*  TODO : when there is no primary key, there is a comma too much at the end of the SQL code ??
	*/
	public String toSQL(int numPk)	
	{
		StringBuffer result = new StringBuffer();
		result.append("     ").append(name).append(" ");			
		
		//TODO : CLOB db2 specific problem
		if (type.equals("VARCHAR")  && Integer.parseInt(size)>255 )
		{
			this.type = "CLOB";			
		}
		//----------------------------------------------------
		
		result.append(type);
		
		if (size != null) {
			result.append("(").append(size);
			if (scale != null) {
				result.append(",").append(scale);
			}
			result.append(") ");
		}
				
		if (isNotNull || isPrimaryKey)
		{
			result.append(" NOT NULL ");
		}				

		if (defaultValue != null)
		{
			result.append("WITH DEFAULT ").append(defaultValue).append('"');
		}

		// Close the column.
		result.append(",\n");

		return result.toString();
	}
	
	public String toDB(int numPk)	
		{
			StringBuffer result = new StringBuffer();
			result.append(" ").append(name).append(" ");			
		
			//TODO : CLOB db specific problem
			if (type.equals("VARCHAR")  && Integer.parseInt(size)>255 )
			{
				this.type = "CLOB";			
			}
			//----------------------------------------------------
		
			result.append(type);
		
			if (size != null) {
				result.append("(").append(size);
				if (scale != null) {
					result.append(",").append(scale);
				}
				result.append(") ");
			}
				
			if (isNotNull || isPrimaryKey)
			{
				result.append(" NOT NULL ");
			}				

			if (defaultValue != null)
			{
				result.append("WITH DEFAULT ").append(defaultValue).append('"');
			}

			// Close the column.
			result.append(",");

			return result.toString();
		}
}
