import java.sql.*;
/**
 *  A database connection class
 */
public class ConnectionDB {

	private String dsn;
	private String driver;
	private String user;
	private String pass;
	private String schemaName;

	private Connection conn;
	private ResultSetMetaData rsetmd;
	private DatabaseMetaData dbmd = null;
	private String catalog;
	private Statement stmt = null;

	/**
	 *  
	 */
	public ConnectionDB(Config connectionSettings){
		//connectionSettings = new Config("connection.properties");
		this.driver = connectionSettings.getProperty("driver");
		this.dsn = connectionSettings.getProperty("dsn");
		this.user = connectionSettings.getProperty("user");
		this.pass = connectionSettings.getProperty("pass");
		this.schemaName = connectionSettings.getProperty("schemaName");
	}
	/**
	 * Open a database connection
	 */
	public void Connect() {
		if (conn != null)
			return;
		try {
			System.out.println("[+] Opening database : " + dsn);
			Class.forName(driver).newInstance();
			conn = DriverManager.getConnection(dsn, user, pass);
			dbmd = conn.getMetaData();
		} catch (ClassNotFoundException ex) {
			System.err.println("[-] Cannot find the database driver classes." + driver);
		} catch (SQLException ex) {
			System.err.println("[-] Cannot connect to this database : " + dsn );
		} catch (Exception ex) {
			System.err.println(ex);
		}
	}				
	
	public Connection getConnection() {
		if (conn == null)
			Connect();
		return (conn);
	}
	/**
	 * Close the database connection
	 */
	public void Close() {
		try {
			if (conn != null) {
				conn.close();
				System.out.println("[+] Closing database");
			}
		} catch (java.sql.SQLException e) {
			System.err.println("[-] Database.closeConnection() - SQLException: " + e);
		}
	}
	
	/**
	 * return the SchemaName associated to this DatabaseConnection
	 * @return
	 */
	public String getSchemaName() {
		return this.schemaName;
	}
	
	/**
	 * execute an SQL query on this database
	 * @param sql = SQL query
	 * @return the resulset associated to the query
	 */
	public DatabaseAnswer Query(String sql) {
		try {
			Connection cnt = getConnection();
			if (cnt == null)
				throw new Exception("[-] Cannot connect to database.");
			Statement st = cnt.createStatement();
			ResultSet rs = st.executeQuery(sql);
			return (new DatabaseAnswer(st, rs));
		} catch (java.sql.SQLException e) {
			System.err.println("[-] FATAL: SQLException " + e);
			System.exit(-1);
		} catch (java.lang.Exception e) {
			System.err.println("[-] FATAL: Exception " + e);
			System.exit(-1);
		}
		return (null);
	}
	/**
	 * execute an SQL create/update/... code
	 * @param sql = SQL DDL action
	 */
	public void Execute(String sql) {
		try {
			Statement stmt = conn.createStatement();
			stmt.execute(sql);
			System.out.println("[+] DDL : " + sql);
		} catch (Exception e) {
			System.err.println("[-] DDL : " + sql );//e.getMessage());
			//e.printStackTrace();
		}
	}
}
