import java.util.List;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Iterator;

public class Database {

	private List tableList = new ArrayList(100);
	private String name;
	private Hashtable tablesByName = new Hashtable();
	private int numberTable = 0;
	private Vector tables;

	/**
	 * Default Constructor
	 */
	public Database() {
		this(null);
	}
	/**
	 * Constructs a table object with a name
	 * @param name table name
	 */
	public Database(String name) {
		this.name = name;
	}
	/**
	 * Get the name of the Database
	 * @return name of the Database
	 */
	public String getName() {
		return name;
	}
	/**
	 * Set the name of the Database
	 * @param name name of the Database
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Return an array of all tables
	 * @return array of all tables
	 */
	public Table[] getTables() {
		int size = tableList.size();
		Table[] tbls = new Table[size];
		for (int i = 0; i < size; i++) {
			tbls[i] = (Table) tableList.get(i);
		}
		return tbls;
	}
	/**
	 * Return the table with the specified name.
	 * @param name table name
	 * @return A Table object. If it does not exist it returns null
	 */
	public Table getTable(String name) {
		return (Table) tablesByName.get(name);
	}

	/**
	 * Add a table to the list and sets the Database property to this Database 
	 * @param tbl  the table to add
	 */
	public void addTable(Table tbl) {
		tbl.setDatabase(this);
		tbl.setID(numberTable);
		numberTable++;
		tableList.add(tbl);
		tablesByName.put(tbl.getName(), tbl);
	}
	/**
	 * Creates a string representation of this Database.
	 * @return string representation
	 */
	public String toSQL(boolean dropTables) throws Exception {
		StringBuffer result = new StringBuffer();
		result.append("-- \n");
		result.append("-- [BEGIN] SQL DB2 \n");
		result.append("-- \n");
		//result.append("USE ").append(getName()).append("\n@\n");
		for (Iterator i = tableList.iterator(); i.hasNext();) {
			Table tbl = (Table) i.next();
			result.append(tbl.toSQL(dropTables));
		}
		
		//get the FK
		for (Iterator i = tableList.iterator(); i.hasNext();) {
			Table tbl = (Table) i.next();
			result.append(tbl.toSQLbis());			
		}

		result.append("-- \n");
		result.append("-- [END] SQL DB2 \n");
		result.append("-- \n");
		return result.toString();
	}
	// BETA TEST
	public void toDB(ConnectionDB conn,boolean dropTables) {
		StringBuffer result = new StringBuffer();
		//get the tables
		try{
		for (Iterator i = tableList.iterator(); i.hasNext();) {
			Table tbl = (Table) i.next();
			tbl.toDB(conn,dropTables);
		}
		//get the FKs
		for (Iterator i = tableList.iterator(); i.hasNext();) {
			Table tbl = (Table) i.next();
			tbl.toDBbis(conn);			
		}
		}catch (Exception ex){
			System.err.println("[-] Database.toDB");			
		}
	}		
}
