import java.util.ArrayList;
import java.util.List;

/**
 * A class for information about foreign keys of a table.
 */
public class ForeignKey
{
    private String foreignTableName;
    private String name;
    private int ID;
    private int tableID;
    private String parentTable;
    private List localColumns = new ArrayList(3);
    private List foreignColumns = new ArrayList(3);
    
	/**
	 * Creates a new column and set the name 
	 * @param name column name
	 */
	public ForeignKey(String source,String foreign,int ID,int tableID)
	{
		this.name = "FK_"+source+"_"+foreign;				
		this.foreignTableName = foreign;
		this.parentTable = source;		
		this.ID = ID;
		this.tableID = tableID;
	}
    
    /**
     * Returns the name 
     */
    public String getName()
    {
        return name;
    }

    /**
     * Sets the name
     */
    public void setName(String name)
    {
        this.name = name;
    }

    /**
     * Get the foreignTableName of the FK
     * @return the name of the foreign table
     */
    public String getForeignTableName()
    {
        return foreignTableName;
    }

    /**
     * Set the foreignTableName of the FK
     * @param tableName the name of the foreign table
     */
    public void setForeignTableName(String tableName)
    {
        foreignTableName = tableName;
    }

    /**
     * Set the parent Table of the foreign key
     * @param parent the table
     */
//    public void setTable(Table parent)
//    {
//       parentTable = parent;
//    }

    /**
     * Get the parent Table of the foreign key
     * @return the parent table
     */
//    public Table getTable()
//    {
//        return parentTable;
//    }

    /**
     * Returns the name of the table the foreign key is in
     * @return the name of the table
     */
//    public String getTableName()
//    {
//        return parentTable.getName();
//    }

    /**
     * Adds a new reference entry to the foreign key
     *
     * @param local name of the local column
     * @param foreign name of the foreign column
     */
    public void addReference(String local, String foreign)
    {
        localColumns.add(local);
        foreignColumns.add(foreign);
    }

    /**
     * Returns the list of local column names. You should not edit this List.
     * @return the local columns
     */
    public List getLocalColumns()
    {
        return localColumns;
    }

    /**
     * Returns the list of foreign column names. You should not edit this List.
     * @return the foreign columns
     */
    public List getForeignColumns()
    {
        return foreignColumns;
    }
	/**
	* Returns a representation of this foreign key.

	* @return representation of this foreign key in SQL
	*/
    public String toSQL() throws Exception
    {
		StringBuffer result = new StringBuffer();
		int counterComma;
		
		result.append("ALTER TABLE " + parentTable + " ADD CONSTRAINT ");
					
		//debug : 27 apr 2004
		//max fk name length = 18 (FK_<tableID>_<ID>)
		String fkIdent = "FK_" + tableID + "_" + ID;		
		if (fkIdent.length() > 18) 
		{
			throw new Exception("FK Constraint name too long"); 
		} else {
			result.append(fkIdent); 
		}

		result.append("\n");
		result.append("     FOREIGN KEY(");		
		counterComma = 0;
		for (int i = 0; i < localColumns.size(); i++)
		{				
			result.append(localColumns.get(i));
			if (localColumns.size() > 1 && counterComma<localColumns.size()-1)
			{
				result.append(",");
				counterComma ++;
			}			   					
		}
				
		result.append(")\n");
		result.append("     REFERENCES "+foreignTableName+"(");
		counterComma = 0;
		for (int i = 0; i < foreignColumns.size(); i++)
		{
			result.append(foreignColumns.get(i));
			if (foreignColumns.size() > 1 && counterComma<foreignColumns.size()-1)
			{
				result.append(",");
				counterComma ++;
			}	
		}				
		result.append(")\n");		
		return result.toString();	
    }
    //BETA TEST
	/**
	* Returns a representation of this foreign key.
	* 
	* @return representation of this foreign key in SQL (WITHOUT \n)
	*/	
    public String toDB() throws Exception
	   {
		   StringBuffer result = new StringBuffer();
		   int counterComma;
		
		   result.append("ALTER TABLE " + parentTable + " ADD CONSTRAINT ");
					
		   //debug : 27 apr 2004
		   //max fk name length = 18 (FK_<tableID>_<ID>)
		   String fkIdent = "FK_" + tableID + "_" + ID;		
		   if (fkIdent.length() > 18) 
		   {
			   throw new Exception("FK Constraint name too long"); 
		   } else {
			   result.append(fkIdent); 
		   }
		   
		   result.append(" FOREIGN KEY(");		
		   counterComma = 0;
		   for (int i = 0; i < localColumns.size(); i++)
		   {				
			   result.append(localColumns.get(i));
			   if (localColumns.size() > 1 && counterComma<localColumns.size()-1)
			   {
				   result.append(",");
				   counterComma ++;
			   }			   					
		   }
				
		   result.append(")");
		   result.append(" REFERENCES "+foreignTableName+"(");
		   counterComma = 0;
		   for (int i = 0; i < foreignColumns.size(); i++)
		   {
			   result.append(foreignColumns.get(i));
			   if (foreignColumns.size() > 1 && counterComma<foreignColumns.size()-1)
			   {
				   result.append(",");
				   counterComma ++;
			   }	
		   }				
		   result.append(")");		
		   return result.toString();	
	   }
}
